from simtek_graphics import *
from numpy import *

# Simulation model
M = 1
x = zeros(2)
v = zeros(2)
dt = 0.1
t = 0.0
radius = 1.0

init_graphics([x], radius, M)

def f_simple(t, x):
  return array([0.0, cos(t)])

# Initial data for f_simple
x[1] = 0.0
v[1] = 0.0

def f_contact(t, x):
  F = zeros(2)

  # Gravity
  F[1] = -1.0

  # Compute wall contact forces using an elastic model (Hooke's law)
  # NB! Here we use x, so this is now a differential equation
  kpen = 10.0
  if x[0] < -width + radius:
    pen = (-width + radius) - x[0]
    F[0] += kpen * pen
  elif x[0] > width - radius:
    pen = (width - radius) - x[0]
    F[0] += kpen * pen
  elif x[1] < -height + radius:
    pen = (-height + radius) - x[1]
    F[1] += kpen * pen
  elif x[1] > height - radius:
    pen = (height - radius) - x[1]
    F[1] += kpen * pen

  return F

# Initial data for f_contact
#x[:] = -4.0
#v[:] = 1.0

# Choose f
f = f_simple

T = 3*2*pi

counter = 0
while(game.running and t < T):

  # Time-stepping
  t0 = t
  x0 = x
  v0 = v

  t = t + dt

  fval0 = f(t0, x0)
  fval = f(t, x)
  v = v0 + 0.5 * dt * fval + 0.5 * dt * fval0
  x = x0 + 0.5 * dt * v + 0.5 * dt * v0

  store(counter, t, [x], [v], [fval])
  update_graphics(counter, [x], dt)
  counter += 1
  
particle_plot(counter)
