from pylab import *

def timestep(f, t0, u0, k, method):
    """Take one time step for the initial value problem (IVP).

    f: function describing IVP u' = f(t, u)
    t0: float for time value f(t0, u0)
    u0: float for initial value
    k: float for time-step value
    method: string naming the time-step method
    
    Returns float u = u0 + k * f(t0, u0)

    Reference test:
    def f(t): return t*t
    timestep(f, 0.1, 1.0, 0.1, "Euler") => 1.001
    """

    t = t0 + k
    if(method == "Euler"):
        u = u0 + k * f(t0, u0)
    elif(method == "Midpoint"):
        u = 0
        # Fill in implementation
    else:
        print "wrong method"
        exit(0)

    return u

def primitive(f, I, u0, k, method):
    """Constructs the primitive function of f by time-stepping

    f: function describing IVP u' = f(t, u)
    I: float array for time interval
    u0: float for initial value
    k: float for time-step value
    method: string naming the time-step method
    
    Returns array of arrays[tarray, uarray] for time values and u values

    Reference test:
    def f(t): return t*t
    [tarray, uarray] = primitive(f, [0, 1], 0, 0.01, "Euler")
    plot(tarray, uarray)
    Expecting the function 1/3*t**3
    """

    uarray = array([])
    tarray = array([])
    t = I[0]
    T = I[1]
    u = u0
    for t in arange(I[0], I[1], k):
        # Fill in implementation
        u = 0
        tarray = append(tarray, t)
        uarray = 0*tarray

    return [tarray, uarray]

def error_est(f, I, u0, k, method):
    """Experimentally computes the difference of |u(T) - ubar(T)| where u(T) is
    computed with time-step k and ubar(T) is computed with time-step k/2.

    f: function describing IVP u' = f(t, u)
    I: float array for time interval
    u0: float for initial value
    k: float for time-step value
    method: string naming the time-step method
    
    Returns float Ek for error estimate for time-step k

    Reference test:
    def f(t): return t*t
    error_est(f, [0, 1], 0, 0.01, "Euler") => 0.0024875
    """

    u1array = [0, 0]
    u2array = [0, 0]
    # Fill in implementation

    Ek = abs(u2array[-1] - u1array[-1])

    return Ek

def convergence_est(f, I, u0, k, method):
    """Experimentally estimates order of convergence of method by halving
    time-step twice

    f: function describing IVP u' = f(t, u)
    I: float array for time interval
    u0: float for initial value
    k: float for time-step value
    method: string naming the time-step method
    
    Returns float Ek for error estimate for time-step k

    Reference test:
    def f(t): return t*t
    [karray, Ekarray] = convergence_est(f, [0, 1], 0, 0.1, "Midpoint")
    plot(log(karray), log(Ekarray))
    Expect slope = 1
    """

    N = 3

    Ekarray = array([])
    karray = array([])
    for i in range(0, N):
        ki = k / pow(2.0, i)
        Ek = error_est(f, I, u0, ki, method)
        Ekarray = append(Ekarray, Ek)
        karray = append(karray, ki)

    return [karray, Ekarray]

def f(t, u):
    return t*t

# Test timestep
u = timestep(f, 0.1, 1.0, 0.1, "Euler")
print u
# Expect 1.001

# Test primitive
[tarray, uarray] = primitive(f, [0, 1], 0, 0.01, "Euler")
subplot(211)
plot(tarray, uarray)

# Test error_est
Ek = error_est(f, [0, 1], 0, 0.01, "Euler")
print Ek
# Expect 0.0024875

# Test convergence_est
[karray, Ekarray] = convergence_est(f, [0, 1], 0, 0.1, "Euler")
subplot(212)
plot(log(karray), log(Ekarray))
show()
