from bisect import *
from pylab import *
import unittest
import fundamental as module


# Utility functions
def near(val, ref, tol):
    return abs(val - ref) <= tol

# Data
def f(t, u):
    return t*t

class Test_Module1(unittest.TestCase):

    # Test timestep
    def test_Timestepping_Question2_Subquestion1(self):
        val = module.timestep(f, 0.1, 1.0, 0.1, "Euler")
        ref = 1.001
        tol = 5

        #print "received: %f reference: %f correct number of decimals required: %f" % (val, ref, tol)

        self.assertAlmostEqual(val,
                               ref,
                               tol,
                               "Time-step with forward Euler too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f correct number of decimals required: %f" % (val, ref, tol))

    # Test primitive
    def test_Primitive_Question3_Subquestion3(self):
        [tarray, uarray] = module.primitive(f, [0, 1], 0, 0.01, "Euler")

        # Data
        def f_primitive(t):
            return t*t*t/3.0

        t0 = 0.0
        i0 = bisect_right(tarray, t0) - 1
        v0 = uarray[i0]
        r0 = 0.0

        t1 = 1.0
        i1 = bisect_right(tarray, t1) - 1
        v1 = uarray[i1]
        r1 = 0.328

        t2 = 0.5
        i2 = bisect_right(tarray, t2) - 1
        v2 = uarray[i2]
        r2 = 0.0429

        tol = 2

        message = "Primitive function with forward Euler too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f correct number of decimals required: %f"

        self.assertAlmostEqual(v0,
                               r0,
                               tol,
                               message % (v0, r0, tol))

        self.assertAlmostEqual(v1,
                               r1,
                               tol,
                               message % (v1, r1, tol))

        self.assertAlmostEqual(v2,
                               r2,
                               tol,
                               message % (v2, r2, tol))

    # Test error_est
    def test_ErrorEstimation_Question4_Subquestion2(self):
        # error_est should be smaller than LT/4*dt
        L = 2
        T = 1
        k = 0.01
        Ek = module.error_est(f, [0, 1], 0, k, "Euler")
        est = L*T/4.0*k

        message = "Error estimate with forward Euler too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f should be between: %f and %f" % (Ek, est, est, 0.2 * est)

        self.failUnless(Ek <= est and Ek > 0.2 * est, message)

    # Test convergence_est
    def test_Convergence_Question5_Subquestion1(self):
        [karray, Ekarray] = module.convergence_est(f, [0, 1], 0, 0.1, "Midpoint")

        p = polyfit(log(karray), log(Ekarray), 1)

        val = p[0]
        ref = 2.0
        tol = 1

        message = "Slope of convergence curve with Midpoint/Trapezoid too far from reference value. See unittest/moduletest.py for details. received: %f reference: %f correct number of decimals required: %f"

        self.assertAlmostEqual(val,
                               ref,
                               tol,
                               message % (val, ref, tol))

unittest.main()
